<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Sebastian Schreiber <breakpoint@schreibersebastian.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

if (!defined('PATH_typo3conf')) {
    die('Could not access this script directly!');
}

require_once(PATH_tslib . 'class.tslib_pagegen.php');
require_once(PATH_tslib . 'class.tslib_fe.php');
require_once(PATH_t3lib . 'class.t3lib_page.php');
require_once(PATH_tslib . 'class.tslib_content.php');
require_once(PATH_t3lib . 'class.t3lib_userauth.php' );
require_once(PATH_tslib . 'class.tslib_feuserauth.php');
require_once(PATH_t3lib . 'class.t3lib_tstemplate.php');
require_once(PATH_t3lib . 'class.t3lib_cs.php');


/**
 *
 * @author Sebastian Schreiber <breakpoint@schreibersebastian.de>
 * @version $Id: class.tx_sschsimpleviewer_eid.php 46 2011-03-20 15:04:10Z schreiberten $
 */
class tx_sschsimpleviewer_eid {

    const CACHE_TYPE = 0;

    /**
     *
     * @var array
     */
    protected $configuration = array();

    /**
     * Init TSFE
     * @return void
     */
    public function __construct() {
        $this->checkCache();
        @set_time_limit(300);
        $this->initTSFE();
    }

    /**
     * Main function of the class. Outputs sitemap.
     *
     * @return	void
     */
    public function main() {
        if ($this->isValidRequest()) {
            $this->setConfiguration();
            
            $cache = tx_sschsimpleviewer_cacheFactory::createCacheFactory(self::CACHE_TYPE);
            $cacheEngine = t3lib_div::makeInstance('tx_sschsimpleviewer_cache', $cache);
            
            $generator = tx_sschsimpleviewer_ViewerFactory::createViewerFactory($this->getGalleryType());
            $generator->injectCacheEngine($cacheEngine);
            $generator->main($this->getConfiguration());            
        } else {
            $this->generateError();
        }
    }

    /**
     * Get the gallery type
     * @return integer
     */
    protected function getGalleryType() {
        $galleryType = intval(t3lib_div::_GP('galleryType'));
        $galleryType = $galleryType ? $galleryType : 0;
        return $galleryType;
    }

    /**
     * Checks wether a request is valid or not
     * @return boolean true or false
     */
    protected function isValidRequest() {
        $galleryId = t3lib_div::_GP('gallery');
        if ($galleryId && t3lib_div::testInt($galleryId)) {
            return true;
        }
        return false;
    }

    /**
     * Check if the files already exists in the cache, it this is the case, get the cached version and stop processing
     * @return void
     */
    protected function checkCache() {
        $cache = tx_sschsimpleviewer_cacheFactory::createCacheFactory(self::CACHE_TYPE);
        /* @var tx_sschsimpleviewer_cache $cacheEngine */
        $cacheEngine = t3lib_div::makeInstance('tx_sschsimpleviewer_cache', $cache);
        $hash = $cacheEngine->getHash();
        if ($cacheEngine->has($hash)) {
            header('Content-type: text/xml');
            echo $cacheEngine->get($hash);
            exit;
        }
        return;
    }

    /**
     * Sends error message.
     *
     * @return	void
     */
    protected function generateError() {
        header('400 Bad Request');
        echo 'Failure. Bad Request';
    }

    /**
     * Get and set the typoscript setup configurations in the class variable configurations
     *
     * @return void
     */
    protected function setConfiguration() {
        $this->configuration = $GLOBALS['TSFE']->tmpl->setup['plugin.']['tx_sschsimpleviewer_pi1.'];        
    }

    /**
     * Get the configuration array
     * @return array
     */
    public function getConfiguration() {
        return $this->configuration;
    }

    /**
     * Initializes TSFE and sets $GLOBALS['TSFE']
     *
     * @return	void
     */
    protected function initTSFE() {
        global $TCA;
        $id = t3lib_div::_GP('id') ? intval(t3lib_div::_GP('id')) : intval(t3lib_div::_GP('page'));        
        $id = $id ? $id : 1;
        $GLOBALS['TSFE'] = t3lib_div::makeInstance('tslib_fe', $GLOBALS['TYPO3_CONF_VARS'], $id, '');

        // $tslib_fe = new tslib_fe();
        $GLOBALS['TSFE']->connectToDB();
        $GLOBALS['TSFE']->initFEuser();
        $GLOBALS['TSFE']->checkAlternativeIdMethods();
        $GLOBALS['TSFE']->determineId();
        $GLOBALS['TSFE']->getCompressedTCarray();
        $GLOBALS['TSFE']->initTemplate();
        $GLOBALS['TSFE']->getConfigArray();

        // Get linkVars, absRefPrefix, etc
        TSpagegen::pagegenInit();
    }

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/class.tx_sschsimpleviewer_eid.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/class.tx_sschsimpleviewer_eid.php']);
}

$generator = t3lib_div::makeInstance('tx_sschsimpleviewer_eid');
/* @var $generator tx_lecubi_eid */
$generator->main();
?>