<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Sebastian Schreiber <breakpoint@schreibersebastian.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */


class tx_sschsimpleviewer_cache implements t3lib_Singleton {

    /**
     *
     * @var 
     */
    protected $cache;
    /**
     *
     * @var integer
     */
    protected $lifetime = 800000;

    public function __construct(tx_sschsimpleviewer_cache_interface $cache = null) {
        if (!$cache) {
            $cache = tx_sschsimpleviewer_cacheFactory::createCacheFactory();
        }

        $this->cache = $cache;
    }


    /**
     * Write content to cache
     * @param string $hash
     * @param string $content
     * @param string $ident
     */
    public function set($hash, $content, $ident) {
        $this->cache->set($hash, $content, array('ident_' . $ident), $this->lifetime);
    }

    /**
     * Write content to cache without the metadata
     * @param string $hash
     * @param string $content
     * @param string $ident
     */
    public function setPure($hash, $content) {
        $this->cache->setPure($hash, $content);
    }

    /**
     * Get content from cache
     * @param string $hash
     * @return string
     */
    public function get($hash) {
        return $this->cache->get($hash);
    }

    /**
     * Create a unique identifier for the request.
     * TODO: What about the Get parameter for the language
     * @return string
     */
    public function getHash() {
        // $lang = intval(t3lib_div::_GET('gallery')) ? intval(t3lib_div::_GET('gallery')) : 0;
        return md5(serialize(array(t3lib_div::getIndpEnv('TYPO3_SITE_URL'), intval(t3lib_div::_GET('gallery')), intval(t3lib_div::_GET('galleryType')))));
    }

    /**
     *
     *
     * @return tx_sschsimpleviewer_cache_interface
     */
    public function getCache() {
        return $this->cache;
    }

    /**
     *
     *
     * @param tx_sschsimpleviewer_cache_interface
     */
    public function setCache($cache) {
        $this->cache = $cache;
    }

    /**
     * Check if a special identifier exists in the cache
     * @param string $hash
     * @return boolean
     */
    public function has($hash) {
        return $this->cache->has($hash);
    }

    /**
     * Remove all from the cache
     */
    public function clearCache() {
        $this->cache->flush();
    }

    /**
     * Set the lifetime for the cache
     * @param integer $lifetime
     */
    public function setLifetime($lifetime) {
        if (!is_integer($lifetime)) {
            throw new InvalidArgumentException('Sorry but this is not an integer ' . gettype($lifetime));
        }
        $this->lifetime = $lifetime;
    }

    public function findCacheFilesByIdentifier($identifier) {
        return $this->cache->findCacheFilesByIdentifier($identifier);
    }


}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/class.tx_sschsimpleviewer_cache.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/class.tx_sschsimpleviewer_cache.php']);
}

