<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Sebastian Schreiber <breakpoint@schreibersebastian.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * @property $cache t3lib_cache_frontend_StringFrontend
 */
class tx_sschsimpleviewer_typo3InternalFileCache implements tx_sschsimpleviewer_cache_interface, t3lib_Singleton {

    /**
     *
     * @var t3lib_cache_frontend_StringFrontend
     */
    protected $cache;

    /**
     *
     * @var string
     */
    protected $fileType;

    public function __construct() {
        if (!is_object($GLOBALS['typo3CacheFactory'])) {
            $this->initCacheManager();
        }
        $this->initCachingFramework();
    }

    /**
     * Initialize the cacheManager
     */
    protected function initCacheManager() {
        $GLOBALS['typo3CacheManager'] = t3lib_div::makeInstance('t3lib_cache_Manager');
        $GLOBALS['typo3CacheFactory'] = t3lib_div::makeInstance('t3lib_cache_Factory');
        $GLOBALS['typo3CacheFactory']->setCacheManager($GLOBALS['typo3CacheManager']);
    }

    /**
     * Initialize the caching framework
     */
    protected function initCachingFramework() {

        try {
            $GLOBALS['typo3CacheFactory']->create(
                    'ssch_simpleviewer_cache',
                    't3lib_cache_frontend_StringFrontend',
                    $GLOBALS['TYPO3_CONF_VARS']['SYS']['caching']['cacheConfigurations']['ssch_simpleviewer_cache']['backend'],
                    $GLOBALS['TYPO3_CONF_VARS']['SYS']['caching']['cacheConfigurations']['ssch_simpleviewer_cache']['options']);
        } catch (Exception $e) {
            throw new t3lib_cache_exception_DuplicateIdentifier($e->getMessage());
        }
        $this->cache = $GLOBALS['typo3CacheManager']->getCache('ssch_simpleviewer_cache');
    }

    /**
     * Write content to cache
     * @param string $hash
     * @param string $content
     * @param string $ident
     */
    public function set($hash, $content, $ident) {
        $this->cache->set($hash, $content, array('ident_' . $ident), $this->lifetime);
    }

    /**
     *
     * @param string $hash
     * @param string $content
     */
    public function setPure($hash, $content) {
        
        $this->cache->remove($hash);

        $temporaryCacheEntryPathAndFilename = $this->cache->getBackend()->getCacheDirectory() . uniqid() . '.temp';
        if (strlen($temporaryCacheEntryPathAndFilename) > t3lib_div::getMaximumPathLength()) {
            throw new t3lib_cache_Exception(
                    'The length of the temporary cache file path "' . $temporaryCacheEntryPathAndFilename .
                    '" is ' . strlen($temporaryCacheEntryPathAndFilename) . ' characters long and exceeds the maximum path length of ' .
                    $this->maximumPathLength . '. Please consider setting the temporaryDirectoryBase option to a shorter path. ',
                    1248710426
            );
        }

        $result = file_put_contents($temporaryCacheEntryPathAndFilename, $content);

        if ($result === FALSE) {
            throw new t3lib_cache_exception(
                    'The temporary cache file "' . $temporaryCacheEntryPathAndFilename . '" could not be written.',
                    1204026251
            );
        }

        $i = 0;
        $cacheEntryPathAndFilename = $this->cache->getBackend()->getCacheDirectory() . $hash . $this->getFileType();
        // @TODO: Figure out why the heck this is done and maybe find a smarter solution, report to FLOW3
        while (!rename($temporaryCacheEntryPathAndFilename, $cacheEntryPathAndFilename) && $i < 5) {
            ++$i;
        }

        // @FIXME: At least the result of rename() should be handled here, report to FLOW3
        if ($result === FALSE) {
            throw new t3lib_cache_exception(
                    'The cache file "' . $cacheEntryPathAndFilename . '" could not be written.',
                    1222361632
            );
        }
    }

    /**
     *
     * @return string
     */
    public function getFileType() {
        if($this->fileType) {
            $fileType = '.' . $this->fileType;
            return $fileType;
        }
        return '';
    }

    /**
     *
     * @param string $fileType
     */
    public function setFileType($fileType) {
        $this->fileType = $fileType;
    }

    /**
     * Get content from cache
     * @param string $hash
     * @return string
     */
    public function get($hash) {
        return $this->cache->get($hash);
    }

    /**
     * Check if a special identifier exists in the cache
     * @param string $hash
     * @return boolean
     */
    public function has($hash) {
        return $this->cache->has($hash);
    }

    /**
     * Remove all from the cache
     */
    public function flush() {
        $this->cache->flush();
    }

    public function findCacheFilesByIdentifier($identifier) {
        $pattern = $this->cache->getBackend()->getCacheDirectory() . $identifier;
        $filesFound = glob($pattern);
        if ($filesFound === FALSE || count($filesFound) === 0) {
            return FALSE;
        }

        $filesFound = array_map('stripPathSite', $filesFound);

        return $filesFound;
    }

}

function stripPathSite($val) {
    return str_replace(PATH_site, '', $val);
}

?>
