<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Sebastian Schreiber <schreiber@springerf3.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

class ViewerAttributeException extends Exception {

}

/**
 *
 * @author Sebastian Schreiber <breakpoint@schreibersebastian.de>
 * @version $Id: class.tx_sschsimpleviewer_AbstractRenderer.php 38 2011-01-27 15:57:38Z schreiberten $
 */
abstract class tx_sschsimpleviewer_AbstractRenderer {

    /**
     * Hold the attributes for the gallery
     * @var array
     */
    protected static $options = array();

    /**
     * Nothing special at the moment
     */
    public function __construct() {
        // just to be sure;
        array_unique(self::$options);
    }

    /**
     * Create start XML tags (including XML prologue) for the viewer.
     *
     * @return	string	Start tags
     */
    abstract public function getStartTags();

    /**
     * Creates end XML tags for this sitemap.
     *
     * @return	string	End XML tags
     */
    abstract public function getEndTags();

    /**
     * Create the attributes for a gallery from the defined array
     * @param array $options
     * @return string
     */
    protected static function createAttributesForGallery(array $options = array()) {
        $attributes = '';
        foreach ($options as $key => $value) {
            $attributes .= ' ' . $key . '="' . htmlspecialchars($value) . '"';
        }
        return $attributes;
    }

    /**
     * Simple getter method
     * @return array
     */
    public function getOptions() {
        return self::$options;
    }

    /**
     * Merges the default options with the given one and creates an attribute string
     * @param array $options
     * @return string
     */
    public function getAttributes(array $addOptions = array()) {
        foreach ($addOptions as $key => $value) {
            if ($this->isValidAttributeKey($key)) {
                continue;
            } else {
                throw new ViewerAttributeException('The option::' .$key);
            }
        }
        return self::createAttributesForGallery(array_merge(self::$options, $addOptions));
    }

    /**
     * Set options in an array
     * @param array $options
     * @return void
     */
    public function setOptions(array $options = array()) {
        foreach ($options as $key => $value) {
            if ($this->isValidAttributeKey($key)) {
                $this->setOption($key, $value);
            }
        }
    }

    /**
     * Set an option of the gallery
     * @param string $name
     * @param mixed $value
     * @return void
     */
    public function setOption($name, $value) {
        if ($this->isValidAttributeKey($name)) {
            self::$options[$name] = $value;
        }
    }

    /**
     * Get a value of the options array
     * @param string $name
     * @return mixed
     */
    public function getOption($name) {
        if ($this->isValidAttributeKey($name)) {
            return self::$options[$name];
        }
    }

    /**
     * Checks if the defined name is a valid attriute of this gallery
     * @param string $name
     * @return boolean true or false
     */
    protected function isValidAttributeKey($name) {
        if (array_key_exists($name, self::$options)) {
            return true;
        }
        return false;
    }

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/lib/Renderers/class.tx_sschsimpleviewer_AbstractRenderer.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/lib/Renderers/class.tx_sschsimpleviewer_AbstractRenderer.php']);
}