<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Sebastian Schreiber <schreiber@springerf3.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 *
 * @author Sebastian Schreiber <breakpoint@schreibersebastian.de>
 * @version $Id: class.tx_sschsimpleviewer_SimpleViewerBasisGalleryXML.php 49 2011-03-24 08:20:56Z schreiberten $
 */

/**
 * @property renderer AbstractRenderer
 * @property cObj tslib_cObj
 */
abstract class tx_sschsimpleviewer_SimpleViewerBasisGalleryXML {

    /**
     *
     * @var array
     */
    protected $properties = array('hasFlickr' => true);
    /**
     *
     * @var string
     */
    protected $sysLanguageMode;
    /**
     * @var integer
     */
    protected $language = 0;
    /**
     * cObject to generate links
     *
     * @var	tslib_cObj
     */
    protected $cObj = null;
    /**
     *
     * @var AbstractRenderer
     */
    protected $renderer = null;
    /**
     *
     * @var string
     */
    protected $path = '';
    /**
     *
     * @var array
     */
    protected $conf = array();
    /**
     * @var options
     */
    protected $options = array();
    /**
     * @var string
     */
    protected $optionsName = '';
    /**
     *
     * @var tx_sschsimpleviewer_cache
     */
    protected $cacheEngine = null;

    /**
     *
     * @param integer $startPid
     * @return void
     */
    public function __construct() {
        /* @var $this->cObj tslib_cObj */
        $this->cObj = t3lib_div::makeInstance('tslib_cObj');
        $this->cObj->start(array());
        // sys_language_mode defines what to do if the requested translation is not found
        $this->sysLanguageMode = $this->conf['sysLanguageMode'] ? $this->conf['sysLanguageMode'] : $GLOBALS['TSFE']->sys_language_mode;
        $this->sysLanguageMode = $this->sysLanguageMode ? $this->sysLanguageMode : 'strict';

        // TODO Make the language Parameter configurable
        $this->language = intval(t3lib_div::_GP('L'));
        $this->language = $this->language ? $this->language : 0;
    }

    /**
     *
     * @param tx_sschsimpleviewer_cache $cacheEngine
     */
    public function injectCacheEngine($cacheEngine) {
        $this->cacheEngine = $cacheEngine;
    }

    /**
     * Parse string through RTE-Functions
     * @param string $str
     * @return string
     */
    public function parseRTE($str) {
        $parseFunc = $GLOBALS['TSFE']->tmpl->setup['lib.']['parseFunc_RTE.'];
        if (is_array($parseFunc)) {
            $str = $this->cObj->parseFunc($str, $parseFunc);
        }
        return $str;
    }

    /**
     * Render the content
     * @param string $content
     */
    public function renderContent($content) {
        $this->writeContentToCache($content);
        // Start
        header('Content-type: text/xml');
        echo $content;
        exit;
    }

    /**
     *
     * @return void
     */
    public function main($conf) {
        $this->setConf($conf);

        $this->parseOptions();
        $this->addOptionsToRenderer();

        $galleryUid = $this->getGalleryUid();
        $gallery = $this->fetchGallery($galleryUid);
        $images = array();
        switch ($gallery['mode']) {
            case 1:
                $images = $this->fetchImagesFromDBForGallery($galleryUid);
                $this->path = 'uploads/tx_sschsimpleviewer/';
                break;
            case 2:
                $images = $this->fetchImagesFromDAMDirectory($gallery['path']);
                $this->path = $gallery['path'];
                break;
            case 3:
                $images = $this->fetchImagesFromDAM($galleryUid);
                break;
            case 4:
                if ($this->properties['hasFlickr']) {
                    $this->renderer->setOption('useFlickr', 'true');
                    if (strlen($gallery['flickrusername']) > 0) {
                        $this->renderer->setOption('flickrUserName', $gallery['flickrusername']);
                    }
                    if (strlen($gallery['flickruid']) > 0) {
                        $this->renderer->setOption('flickrUserId', $gallery['flickruid']);
                    }
                } else {
                    throw new Exception('The Flickr Mode is not allowed for this gallerytype');
                }
                break;
            case 5:
                $images = $this->fetchImagesFromDAMCategory($galleryUid);
                break;
            default:
                $images = $this->fetchImagesFromDirectoryForGallery($gallery['path']);
                $this->path = $gallery['path'];
                break;
        }

        if (is_array($images) && is_array($gallery)) {
            $this->renderXMLForImages($images, $gallery);
        } else {
            throw new ViewerAttributeException('Either ' . $images . ' or ' . $gallery . ' is not an array');
        }
    }

    /**
     * Get images from dam categories
     * @param integer $galleryUid
     * @return array
     */
    protected function fetchImagesFromDAMCategory($galleryUid = 0) {
        $images = array();
        if ($galleryUid && t3lib_div::testInt($galleryUid)) {
            // Hole alle Bilder von der Kategorie die bei der definierten Galerie eingetragen ist und das rekursiv
            $table = 'tx_sschsimpleviewer_gallery_dam_cat';
            $where = 'uid_local = ' . $galleryUid;
            $fields = '*';
            $cats = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows($fields, $table, $where);
            if (is_array($cats) && !empty($cats)) {
                $catListArray = array();
                foreach ($cats as $cat) {
                    array_push($catListArray, $cat['uid_foreign']);
                }
                $catList = implode(',', $catListArray);

                unset($cats);
                unset($catListArray);

                $listRecursive = $this->getRecursiveDamCat($catList, 99);
                $listArray = explode(',', $listRecursive);
                foreach ($listArray as $cat) {
                    // add images from categories
                    $fields = 'tx_dam.alt_text as link, CONCAT(tx_dam.file_path,tx_dam.file_name) as image, tx_dam.uid, tx_dam.title as caption, tx_dam.description';
                    $tables = 'tx_dam, tx_dam_mm_cat';
                    $temp_where = 'tx_dam.deleted = 0 AND tx_dam.file_mime_type=\'image\' AND tx_dam.hidden=0 AND tx_dam_mm_cat.uid_foreign=' . $cat . ' AND tx_dam_mm_cat.uid_local=tx_dam.uid';
                    $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery($fields, $tables, $temp_where, '', 'tx_dam.sorting');
                    while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                        $images[] = $row; # just add the image to an array
                    }
                    $GLOBALS['TYPO3_DB']->sql_free_result($res);
                }
            }
        }
        return $images;
    }

    /**
     * Get the gallery uid from get-vars
     * @return integer
     */
    protected function getGalleryUid() {
        $galleryUid = intval(t3lib_div::_GP('gallery'));
        $galleryUid = $galleryUid ? $galleryUid : intval($this->conf['galleryUid']);
        $galleryUid = $galleryUid ? $galleryUid : 1;
        return $galleryUid;
    }

    /**
     * Get the renderer
     * @return AbstractRenderer
     */
    public function getRenderer() {
        return $this->renderer;
    }

    /**
     * Set configuration for the gallery
     * @param array $conf
     */
    public function setConf($conf) {
        $this->conf = $conf;
    }

    /**
     * Writes the content to the cache
     */
    protected function writeContentToCache($content) {
        if ($this->cacheEngine) {
            $hash = $this->cacheEngine->getHash();
            $this->cacheEngine->set($hash, $content, __FUNCTION__);
        }
    }

    /**
     * Parse different options for the gallery defined via Typoscript
     * @return void
     */
    public function parseOptions() {
        if (isset($this->conf[$this->optionsName . '.'])) {
            $optionsArray = t3lib_div::trimExplode(',', $this->conf[$this->optionsName . '.']['attributes']);
            foreach ($optionsArray as $key => $option) {
                $optionArray = t3lib_div::trimExplode(':', $option);
                $this->options[$optionArray[0]] = $optionArray[1];
                unset($optionArray);
            }
            unset($optionsArray);
        }
    }

    /**
     * Set the options for the renderer
     * @return void
     */
    public function addOptionsToRenderer() {
        if ($this->renderer) {
            $this->renderer->setOptions($this->options);
        }
    }

    /**
     *
     * @param array $images
     * @param array $gallery
     * @return void
     */
    abstract protected function renderXMLForImages(array $images, array $gallery);

    /**
     * Create the big image of the gallery
     * @param string $image
     * @return string
     */
    protected function createBigImage($image) {

        $conf = array();
        $conf['file'] = $image;
        $conf['file.']['maxW'] = isset($this->conf['maxImageWidth']) ? $this->conf['maxImageWidth'] : 640;
        $conf['file.']['maxH'] = isset($this->conf['maxImageHeight']) ? $this->conf['maxImageWidth'] : 640;
        return t3lib_div::getIndpEnv('TYPO3_SITE_URL') . $this->cObj->IMG_RESOURCE($conf);
    }

    /**
     * Create the small image of the gallery
     * @param string $image
     * @return string
     */
    protected function createThumbImage($image) {
        $conf = array();
        $conf['file'] = $image;
        $conf['file.']['maxW'] = isset($this->conf['maxThumbImageWidth']) ? $this->conf['maxThumbImageWidth'] : 200;
        $conf['file.']['maxH'] = isset($this->conf['maxThumbImageHeight']) ? $this->conf['maxThumbImageHeight'] : 200;
        return t3lib_div::getIndpEnv('TYPO3_SITE_URL') . $this->cObj->IMG_RESOURCE($conf);
    }

    /**
     *
     * @param integer $galleryUid
     * @return array
     */
    protected function fetchGallery($galleryUid = 0) {
        $rows = array();
        if (t3lib_div::testInt($galleryUid) && $galleryUid) {
            t3lib_div::loadTCA('tx_sschsimpleviewer_gallery');
            $where = 'uid = ' . $galleryUid;
            $where .= $this->cObj->enableFields('tx_sschsimpleviewer_gallery');
            $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'tx_sschsimpleviewer_gallery', $where);
            if ($res) {
                while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                    if ($this->getLanguage()) {
                        $row = $GLOBALS['TSFE']->sys_page->getRecordOverlay('tx_sschsimpleviewer_gallery', $row, $this->getLanguage(), $this->getOLMode());
                    }
                    $rows[] = $row;
                }
            } else {
                throw new Exception('Sorry, but something went wrong with your DB-Request');
            }
            $GLOBALS['TYPO3_DB']->sql_free_result($res);
        }
        return $rows[0];
    }

    /**
     *
     * @param integer $galleryUid
     * @return array
     */
    protected function fetchImagesFromDBForGallery($galleryUid = 0) {
        $rows = array();
        if (t3lib_div::testInt($galleryUid) && $galleryUid) {
            t3lib_div::loadTCA('tx_sschsimpleviewer_images');
            $where = 'parentid = ' . $galleryUid;
            $where .= $this->cObj->enableFields('tx_sschsimpleviewer_images');
            $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 'tx_sschsimpleviewer_images', $where);

            if ($res) {
                while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                    if ($this->getLanguage()) {
                        $row = $GLOBALS['TSFE']->sys_page->getRecordOverlay('tx_sschsimpleviewer_images', $row, $this->getLanguage(), $this->getOLMode());
                    }
                    $rows[] = $row;
                }
            } else {
                throw new Exception('Sorry, but something went wrong with your DB-Request');
            }
            $GLOBALS['TYPO3_DB']->sql_free_result($res);
        }
        return $rows;
    }

    /**
     *
     * @param string $path
     * @return array
     */
    public function fetchImagesFromDirectoryForGallery($path = '') {
        if ($path) {

            try {
                $simplePath = $path;
                // Due to open_basedir restrictions on shared hosting servers.
                $path = realpath($path);
                $path = $path ? $path : t3lib_div::getIndpEnv('TYPO3_DOCUMENT_ROOT') . DIRECTORY_SEPARATOR . $simplePath;
                $it = new RecursiveDirectoryIterator($path);
                $images = array();
                foreach (new RecursiveIteratorIterator($it) as $file) {

                    if ($this->isFileAllowed($file)) {
                        $image = array();
                        $image['image'] = $prefix . basename($file);
                        $image['orig'] = basename($file);
                        $image['caption'] = basename($file);
                        $image['description'] = basename($file);
                        // maybe you want to call clearstatcache.
                        $image['stats'] = stat($file);
                        array_push($images, $image);
                    }
                }
                usort($images, 'sortBy' . ucfirst(str_replace('_', '', $this->getOrderBy())) . strtoupper($this->getOrderByDirection()));
                return $images;
            } catch (Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     *
     * @param string $file
     * @return boolean
     */
    public function isFileAllowed($file) {
        $extension = $this->getFileExtension($file);
        return array_search($extension, array('jpg', 'jpeg', 'png', 'gif')) !== FALSE ? TRUE : FALSE;
    }

    /**
     * Get the extension of the file
     * @param string $filename
     * @return string
     */
    public function getFileExtension($filename) {
        return strtolower(substr(strrchr($filename, '.'), 1));
    }

    /**
     * Fetch the images from a defined dam directory
     * @param string $path
     * @return array
     */
    public function fetchImagesFromDAMDirectory($path) {
        $orderBy = $this->getOrderBy() . ' ' . $this->getOrderByDirection();
        $images = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows('alt_text as link, title as caption, description, file_name as image', 'tx_dam', 'deleted = 0 AND hidden = 0 AND file_mime_type = "image" AND file_path LIKE "%' . $GLOBALS['TYPO3_DB']->escapeStrForLike($path, 'tx_dam') . '%"', '', $orderBy);
        return $images;
    }

    /**
     * Fetch a record from the dam by uid
     * @param integer $uid
     * @return array
     */
    public function fetchImagesFromDAM($uid) {
        $images = tx_dam_db::getReferencedFiles('tx_sschsimpleviewer_gallery', $uid, 'simpleviewer', 'tx_dam_mm_ref', 'tx_dam.alt_text as link, CONCAT(tx_dam.file_path,tx_dam.file_name) as image, tx_dam.uid, tx_dam.title as caption, tx_dam.description');
        return $images['rows'];
    }

    /**
     * Get a list of recursive categories
     *
     * @param	string		$id: comma seperated list of ids
     * @param	int		$level: the level for recursion
     * @return	image(s)
     */
    protected function getRecursiveDamCat($id, $level = 0) {
        $result = $id . ','; # add id of 1st level
        $idList = explode(',', $id);

        if ($level > 0) {
            $level--;

            foreach ($idList as $key => $value) {
                $where = 'hidden = 0 AND deleted = 0 AND parent_id = ' . $id;
                $res = $GLOBALS['TYPO3_DB']->exec_SELECTquery('uid', 'tx_dam_cat', $where);
                while ($row = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($res)) {
                    $rec = $this->getRecursiveDamCat($row['uid'], $level);
                    if ($rec != '') {
                        $result .= $rec . ',';
                    }
                }
            }
        }
        $result = str_replace(',,', ',', $result);
        $result = substr($result, 0, -1);
        return $result;
    }

    /**
     * Check if the file exists
     * @param string $file
     * @return boolean
     */
    public function thumbFileExists($file = '') {
        $thumbFile = $this->path . 'thumbs/' . basename($file);
        if (is_file($thumbFile) && file_exists($thumbFile)) {
            return $thumbFile;
        }
        return false;
    }

    /**
     * A little helper function to get the configured OL-Mode
     *
     * @return	string:		The OL-Mode defined via typoscript
     */
    protected function getOLMode() {
        $olMode = ($this->sysLanguageMode == 'strict' ? 'hideNonTranslated' : '');
        return $olMode;
    }

    /**
     * Get the language code
     * @return string
     */
    protected function getLanguage() {
        return $this->language;
    }

    /**
     * Get the orderby setting
     * @return string
     */
    protected function getOrderBy() {
        return $this->conf['orderBy'] ? $this->conf['orderBy'] : 'file_ctime';
    }

    /**
     * Get the orderby direction setting
     * @return string
     */
    protected function getOrderByDirection() {
        return $this->conf['orderByDirection'] ? $this->conf['orderByDirection'] : 'desc';
    }

    /**
     * Create a link to the image
     * @param integer $pageId
     * @return link
     */
    protected function getLinkToPage($pageId = 0) {
        if ($this->cObj && $pageId) {
            $conf = array(
                'parameter' => $pageId,
                'returnLast' => 'url',
                'useCacheHash' => true,
            );
            $link = htmlspecialchars($this->cObj->typoLink('', $conf));
            return t3lib_div::locationHeaderUrl($link);
        }
    }

}

function sortByNameDesc($file1, $file2) {
    return ($file1['image'] < $file2['image']);
}

function sortByNameAsc($file1, $file2) {
    return ($file1['image'] > $file2['image']);
}

function sortByFilectimeDesc($file1, $file2) {
    return ($file1['stats']['ctime'] < $file2['stats']['ctime']);
}

function sortByFilectimeAsc($file1, $file2) {
    return ($file1['stats']['ctime'] > $file2['stats']['ctime']);
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/lib/class.tx_sschsimpleviewer_SimpleViewerBasisGalleryXML.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/lib/class.tx_sschsimpleviewer_SimpleViewerBasisGalleryXML.php']);
}