<?php

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2010 Sebastian Schreiber <me@schreibersebastian.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

require_once(PATH_tslib . 'class.tslib_pibase.php');

/**
 *
 * @author Sebastian Schreiber <breakpoint@schreibersebastian.de>
 * @version $Id: class.tx_sschsimpleviewer_pi1.php 49 2011-03-24 08:20:56Z schreiberten $
 */
class tx_sschsimpleviewer_pi1 extends tslib_pibase {

    /**
     * Same as class name
     * @var string
     */
    public $prefixId = 'tx_sschsimpleviewer_pi1';
    /**
     * Path to this script relative to the extension dir.
     * @var string
     */
    public $scriptRelPath = 'pi1/class.tx_sschsimpleviewer_pi1.php';
    /**
     * The extension key.
     * @var string
     */
    public $extKey = 'ssch_simpleviewer';
    /**
     * The template File
     * @var string
     */
    private $template = '';
    /**
     *
     * @var boolean
     */
    public $pi_checkCHash = true;
    /**
     *
     * @var booelan
     */
    public $pi_USER_INT_obj = false;

    /**
     * The main method of the PlugIn
     *
     * @param	string		$content: The PlugIn content
     * @param	array		$conf: The PlugIn configuration
     * @return	The content that is displayed on the website
     */
    public function main($content, $conf) {
        $this->conf = $conf;
        $this->pi_setPiVarDefaults();
        $this->pi_loadLL();
        // call init-Function for flexform initilization
        if (!isset($this->conf['displayCode'])) {
            $content = $this->pi_getLL('ll_no_ts_template_defined', 'Please, install the typoscript template at first under Includes.');
        } else {
            $this->init();
            if (isset($this->conf['gallery']) && !empty($this->conf['gallery'])) {
                switch (strtolower($this->conf['displayCode'])) {
                    case 'gallery':
                        $content = $this->renderGallery();
                        break;
                    case 'gallerylink':
                        $content = $this->renderGalleryLink();
                        break;
                }
            } else {
                $content = $this->pi_getLL('ll_no_gallery_defined', 'Please, create and define a gallery first.');
            }
        }

        return $this->baseWrap($content);
    }

    /**
     * Render the gallery content
     * @return string
     */
    protected function renderGallery() {
        $viewer = 'Simpleviewer';

        switch ($this->conf['galleryType']) {
            case 1:
                $viewer = 'SimpleViewer';
                break;
            case 2:
                // Tiltviewer
                $viewer = 'Tiltviewer';
                break;
            case 3:
                // Autoviewer
                $viewer = 'Autoviewer';
                break;
            case 4:
                // PostcardViewer
                $viewer = 'Postcardviewer';
                break;
        }
        return $this->renderGalleryContent($viewer);
    }

    /**
     * Render the links to the gallery
     * @return string
     */
    protected function renderGalleryLink() {
        // init the content var
        $content = '';
        $gallerySubpart = $this->cObj->getSubpart($this->template, '###SIMPLEVIEWER_GALLERYLINK_TEMPLATE###');
        $galleryMarkerArray = array();
        $galleryMarkerArray['###LINK###'] = $this->getGalleryLink();
        $galleryMarkerArray['###SITE_REL_PATH###'] = t3lib_extMgm::siteRelPath($this->extKey);
        $galleryMarkerArray['###LINKTEXT###'] = $this->conf['galleryLinktext'] ? $this->conf['galleryLinktext'] : $this->pi_getLL('ll_to_gallery', 'Zur Bildergalerie');
        $content .= $this->cObj->substituteMarkerArrayCached($gallerySubpart, $galleryMarkerArray);
        return $content;
    }

    /**
     * Get the link to the gallery
     * @return string
     */
    protected function getGalleryLink() {
        global $TYPO3_CONF_VARS;

        $linkParams = array();
        $linkParams['gallery'] = $this->conf['gallery'];
        if ($this->conf['galleryType']) {
            $linkParams['galleryType'] = $this->conf['galleryType'];
        }

        $linkParams = array_merge($linkParams, $this->getLangParam());
        $conf = array();
        // TODO Make caching configurable
        $conf['useCacheHash'] = $this->pi_checkCHash;

        // Many thanks to Veronica Francas from comvos.de http://bugs.typo3.org/view.php?id=4564
        $extConfig = unserialize($TYPO3_CONF_VARS['EXT']['extConf'][$this->extKey]);
        $conf['parameter'] = $GLOBALS['TSFE']->id . ',' . $extConfig['typeNum'];
        $conf['returnLast'] = true;
        $conf['additionalParams'] = t3lib_div::implodeArrayForUrl('', $linkParams);
        return $this->cObj->typoLink('', $conf);
    }

    /**
     * Get the apprpriate lang parameter
     * @return array
     */
    protected function getLangParam() {
        $linkParams = array();
        if ($GLOBALS['TSFE']->sys_language_content) {
            // TODO Make language Parameter configurable
            $linkParams['L'] = $GLOBALS['TSFE']->sys_language_content;
        }
        return $linkParams;
    }

    /**
     * Render the gallery. Which type of gallery depends on the view parameter string
     * @param string $view
     * @return string
     */
    protected function renderGalleryContent($view = 'simpleviewer') {
        // init the content var
        $content = '';
        $subparts = array();
        $subpart = '###' . strtoupper(trim($view)) . '_GALLERY_TEMPLATE###';
        $gallerySubpart = $this->cObj->getSubpart($this->template, $subpart);

        $galleryMarkerArray = array();
        $galleryMarkerArray = $this->initLLMarkerArray($gallerySubpart);
        $galleryMarkerArray['###GALLERY_RAW###'] = $this->getGalleryEidLink();
        $galleryId = $this->getGalleryId();
        $galleryMarkerArray['###GALLERY_ID###'] = $galleryId;
        $galleryMarkerArray['###MAX_IMAGE_WIDTH###'] = $this->conf['maxImageWidth'] ? $this->conf['maxImageWidth'] : '640';
        $galleryMarkerArray['###VIEWER_SWF###'] = $this->getPathToViewer($view);

        $flickruid = $this->getUseFlickr();
        if (!is_null($flickruid)) {
            $galleryMarkerArray['###USE_FLICKR###'] = 'true';
            $galleryMarkerArray['###FLICKR_USER_ID###'] = strlen($flickruid) > 0 ? htmlspecialchars($flickruid) : htmlspecialchars($this->conf['flickr_user_id']);
        } else {
            $galleryMarkerArray['###USE_FLICKR###'] = 'false';
            $galleryMarkerArray['###FLICKR_USER_ID###'] = '';
        }

        if ($this->isMobileSupported()) {

            // Get the static xml gallery path for the mobile version. A dynamic url doesn´t work correctly
            $mobileContent = t3lib_div::getURL($this->getGalleryEidLink());
            $cache = tx_sschsimpleviewer_cacheFactory::createCacheFactory();
            $cacheEngine = t3lib_div::makeInstance('tx_sschsimpleviewer_cache', $cache);
            $hash = md5($this->getGalleryEidLink());

            try {
                $cache->setFileType('xml');
                $cacheEngine->setPure($hash, $mobileContent);
                $galleryXml = $cacheEngine->findCacheFilesByIdentifier($hash . $cache->getFileType());
                if (is_array($galleryXml) && !empty($galleryXml)) {
                    // JS-Part
                    $mobileJSSupportSubpart = $this->cObj->getSubpart($gallerySubpart, '###USE_MOBILE_VERSION###');
                    $mobileJSSupportArray = array();
                    $mobileJSSupportArray = $this->initLLMarkerArray($mobileJSSupportSubpart);
                    $mobileJSSupportArray['###SVCOREURL###'] = $this->conf['svCoreUrl'];
                    $mobileJSSupportArray['###SITE_REL_PATH###'] = t3lib_extMgm::siteRelPath($this->extKey);
                    $subparts['###USE_MOBILE_VERSION###'] = $this->cObj->substituteMarkerArray($mobileJSSupportSubpart, $mobileJSSupportArray);

                    // Embed Part
                    $mobileJSSupportEmbedSubpart = $this->cObj->getSubpart($gallerySubpart, '###USE_MOBILE_VERSION_EMBED###');
                    $mobileJSEmbedArray = array();
                    $mobileJSEmbedArray = $this->initLLMarkerArray($mobileJSSupportEmbedSubpart);
                    $mobileJSEmbedArray['###GALLERY_ID###'] = $galleryId;
                    $galleryMarkerArray['###GALLERY###'] = $galleryXml[0];
                    $subparts['###USE_MOBILE_VERSION_EMBED###'] = $this->cObj->substituteMarkerArray($mobileJSSupportEmbedSubpart, $mobileJSEmbedArray);

                    // Clear the Non-Mobile Subpart
                    $subparts['###USE_NON_MOBILE_VERSION_EMBED###'] = '';
                    $subparts['###USE_NON_MOBILE_VERSION###'] = '';
                } else {
                    throw new Exception('You can´t use the mobile version with a non-cache system. Please set the TS-Setting useMobileVersion to false or 0');
                }
            } catch (Exception $e) {
                echo $e->getMessage();
                exit;
            }
        } else {
            $galleryMarkerArray['###GALLERY###'] = $this->getGalleryEidLinkRawUrlEncoded();

            $nonMobileJSSupportSubpart = $this->cObj->getSubpart($gallerySubpart, '###USE_NON_MOBILE_VERSION###');
            $nonMobileJSSupportArray = array();
            $nonMobileJSSupportArray = $this->initLLMarkerArray($nonMobileJSSupportSubpart);
            $nonMobileJSSupportArray['###SITE_REL_PATH###'] = t3lib_extMgm::siteRelPath($this->extKey);
            $subparts['###USE_NON_MOBILE_VERSION###'] = $this->cObj->substituteMarkerArray($nonMobileJSSupportSubpart, $nonMobileJSSupportArray);

            $nonMobileJSSupportEmbedSubpart = $this->cObj->getSubpart($gallerySubpart, '###USE_NON_MOBILE_VERSION_EMBED###');
            $nonMobileJSEmbedArray = array();
            $nonMobileJSEmbedArray = $this->initLLMarkerArray($nonMobileJSSupportEmbedSubpart);
            $nonMobileJSEmbedArray['###GALLERY###'] = $this->getGalleryEidLinkRawUrlEncoded();
            $nonMobileJSEmbedArray['###GALLERY_ID###'] = $galleryId;
            $nonMobileJSEmbedArray['###VIEWER_SWF###'] = $this->getPathToViewer($view);
            $subparts['###USE_NON_MOBILE_VERSION_EMBED###'] = $this->cObj->substituteMarkerArray($nonMobileJSSupportEmbedSubpart, $nonMobileJSEmbedArray);

            // Clear the Mobile Subparts
            $subparts['###USE_MOBILE_VERSION###'] = '';
            $subparts['###USE_MOBILE_VERSION_EMBED###'] = '';
        }



        return $this->substituteMarkersAndSubparts($gallerySubpart, $galleryMarkerArray, $subparts);
    }

    /**
     * Get the path to the viewer file
     * @return string
     */
    protected function getPathToViewer($view = 'simpleviewer') {
        $viewer = strtolower($view);
        $viewerSwf = $viewer;
        $pathToViewer = !empty($this->conf[$viewerSwf]) ? $this->conf[$viewerSwf] : 'typo3conf/ext/ssch_simpleviewer/res/simpleviewer/' . $viewer . '.swf';
        return $pathToViewer;
    }

    /**
     * Replaces $this->cObj->substituteArrayMarkerCached() because substitued
     * function polutes cache_hash table a lot.
     *
     * @param	string		$template	Template
     * @param	array		$markers	Markers
     * @param	array		$subparts	Subparts
     * @return	string		HTML
     */
    protected function substituteMarkersAndSubparts($template, array $markers, array $subparts) {
        $content = $this->cObj->substituteMarkerArray($template, $markers);
        if (count($subparts) > 0) {
            foreach ($subparts as $name => $subpart) {
                $content = $this->cObj->substituteSubpart($content, $name, $subpart);
            }
        }
        return $content;
    }

    /**
     * Check if the current galleryType supports the mobile version
     * @return boolean
     */
    public function isMobileSupported() {
        if ($this->conf['useProVersion'] && $this->conf['useMobileVersion'] && $this->conf['galleryType'] == 1) {
            return true;
        }
        return false;
    }

    /**
     * Use flickr or not
     * @return string
     */
    protected function getUseFlickr() {
        $row = $GLOBALS['TYPO3_DB']->exec_SELECTgetRows('mode, flickruid', 'tx_sschsimpleviewer_gallery', 'uid = ' . $this->conf['gallery']);
        if ($row[0]['mode'] == 4) {
            return $row[0]['flickruid'];
        }
        return;
    }

    /**
     * Return the eID-Scripted Link for the Gallery
     * @return string
     */
    protected function getGalleryEidLink(array $addParams = array()) {
        $params = array();
        $params['eID'] = $this->extKey;
        $params['gallery'] = $this->conf['gallery'];

        if ($this->conf['galleryType']) {
            $params['galleryType'] = $this->conf['galleryType'];
        }
        $params['svCoreUrl'] = $this->conf['svCoreUrl'];
        $params['page'] = $GLOBALS['TSFE']->id;

        $params = array_merge($addParams, $params, $this->getLangParam());

        $url = t3lib_div::getIndpEnv('TYPO3_SITE_URL') . $this->pi_getPageLink($GLOBALS['TSFE']->id, '', $params);

        return $url;
    }

    /**
     * Get the link substitute by rawurlencode
     * @param array $addParams
     * @return string
     */
    public function getGalleryEidLinkRawUrlEncoded(array $addParams = array()) {
        return urlencode($this->getGalleryEidLink($addParams));
    }

    /**
     * Get a unique id for the gallery
     * @return string
     */
    protected function getGalleryId() {
        return $this->prefixId . md5($this->template) . '_' . time();
    }

    /**
     * Different BaseWrap
     *
     * @param string $content
     * @return string
     */
    protected function baseWrap($content) {
        if (isset($this->conf['baseWrap.'])) {
            return $this->cObj->stdWrap($content, $this->conf['baseWrap.']);
        } else {
            return $this->pi_wrapInBaseClass($content);
        }
    }

    /**
     * Init Flexform. Fetch all Flexform-Data and defined the global TemplateFile
     *
     * @return	void
     */
    protected function init() {
        // init Flexform
        $this->pi_initPIflexForm();

        // get the global Template
        $templateFile = !empty($this->conf['templateFile']) ? $this->conf['templateFile'] : 'EXT:' . $this->extKey . '/res/tmpl/simpleviewer.html';
        $this->template = $this->cObj->fileResource($templateFile);

        $this->conf['galleryLinktext'] = htmlspecialchars($this->fetchConfigurationValue('galleryLinktext', 'gallery'));

        if (isset($this->conf['galleryLinktext'][0])) {
            $this->conf['displayCode'] = 'gallerylink';
        } else {
            $this->conf['displayCode'] = 'gallery';
        }
        $this->conf['gallery'] = intval(t3lib_div::_GP('gallery') ? t3lib_div::_GP('gallery') : $this->fetchConfigurationValue('gallerySelection', 'gallery'));
        $this->conf['galleryType'] = intval(t3lib_div::_GP('galleryType') ? t3lib_div::_GP('galleryType') : $this->fetchConfigurationValue('galleryType', 'gallery'));

        $this->conf['svCoreUrl'] = strlen($this->conf['svCoreUrl']) > 0 ? $this->conf['svCoreUrl'] : t3lib_extMgm::siteRelPath($this->extKey) . 'res/simpleviewer/';
        // set the header parts like css and javascript. You can configure this over the template-file
        $this->setHeaderParts();
    }

    /**
     * Set the header parts defined in the main template. Output js or stylesheet
     *
     * @return	void
     */
    protected function setHeaderParts() {
        $key = 'EXT:' . $this->extKey . '_' . md5($this->template);
        if (!isset($GLOBALS['TSFE']->additionalHeaderData[$key])) {
            $headerParts = '';
            $headerPartsGlobally = $this->cObj->getSubpart($this->template, '###HEADER_ADDITIONS_' . strtoupper($this->conf['displayCode']) . '###');
            if ($headerPartsGlobally) {
                $headerParts .= $this->substituteHeaderParts($headerPartsGlobally);
            }
            $headerPartsLocally = $this->cObj->getSubpart($this->template, '###HEADER_ADDITIONS_' . strtoupper($this->conf['galleryType']) . '###');
            if ($headerPartsLocally) {
                $headerParts .= $this->substituteHeaderParts($headerPartsLocally);
            }
            $GLOBALS['TSFE']->additionalHeaderData[$key] = $headerParts;
        }
    }

    /**
     * Create the headerparts like JS or CSS
     * @param string $headerParts
     * @return string
     */
    protected function substituteHeaderParts($headerParts) {
        $markerArray = array();
        $markerArray = $this->initLLMarkerArray($headerParts);
        $markerArray['###SITE_REL_PATH###'] = t3lib_extMgm::siteRelPath($this->extKey);
        return $this->cObj->substituteMarkerArray($headerParts, $markerArray);
    }

    /**
     * Fetches the configuration property depending on the given name
     * If no Flexform-Data is defined, we will return at least a default value defined in typoscript
     *
     * @param	string		$param
     * @param	string		$parentParam
     * @return	string		$value: The value of the conf-Property
     */
    protected function fetchConfigurationValue($param, $parentParam) {
        $value = '';
        if ($param && $parentParam) {
            $langId = intval($GLOBALS['TSFE']->config['config']['sys_language_uid']);
            $langDefinition = ($langId != 0) ? 'l' . strtoupper($this->LLkey) : 'lDEF';
            $value = trim($this->pi_getFFvalue($this->cObj->data['pi_flexform'], $param, $parentParam, $langDefinition));
        }
        return $value ? $value : $this->conf[$param];
    }

    /**
     * Init Marker Array for language fields
     *
     * @param	string		$templateCode:
     * @param	string		$row:
     * @return	array		$markerArray:
     */
    public function initLLMarkerArray($templateCode) {

        $markerArray = array();

        if (preg_match_all("/###([_A-Za-z0-9]+?)###/", $templateCode, $regs, PREG_PATTERN_ORDER)) {
            foreach ($regs[0] as $value) {
                if (!strncasecmp('###LL', $value, 5)) {
                    $str = strtolower(str_replace('###', '', $value));
                    $markerArray[$value] = $this->pi_getLL($str);
                }
            }
            return $markerArray;
        }
    }

    /**
     * Injection of the tslib_cObj. Mainly for testing purposes
     * @param tslib_cObj $cObj 
     */
    public function setCobj(tslib_cObj $cObj) {
        $this->cObj = $cObj;
    }

}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/pi1/class.tx_sschsimpleviewer_pi1.php']) {
    include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/ssch_simpleviewer/pi1/class.tx_sschsimpleviewer_pi1.php']);
}
?>