<?php

/* * *************************************************************
 * Copyright notice
 *
 * (c) 2009 Sebastian Schreiber (schreiber@springerf3.de)
 * All rights reserved
 *
 * This script is part of the TYPO3 project. The TYPO3 project is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 *
 * This script is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 *
 * @author Sebastian Schreiber <breakpoint@schreibersebastian.de>
 * @version $Id: ssch_simpleviewer_viewersconfiguration_testcase.php 26 2010-12-04 13:17:50Z schreiberten $
 */
t3lib_div::requireOnce(t3lib_extMgm::extPath('ssch_simpleviewer') . 'lib' . DIRECTORY_SEPARATOR . 'Renderers' . DIRECTORY_SEPARATOR . 'SimpleViewerGalleryRenderer.php');
t3lib_div::requireOnce(t3lib_extMgm::extPath('ssch_simpleviewer') . 'lib' . DIRECTORY_SEPARATOR . 'Renderers' . DIRECTORY_SEPARATOR . 'TiltViewerGalleryRenderer.php');
t3lib_div::requireOnce(t3lib_extMgm::extPath('ssch_simpleviewer') . 'lib' . DIRECTORY_SEPARATOR . 'Renderers' . DIRECTORY_SEPARATOR . 'PostcardViewerGalleryRenderer.php');
t3lib_div::requireOnce(t3lib_extMgm::extPath('ssch_simpleviewer') . 'lib' . DIRECTORY_SEPARATOR . 'Renderers' . DIRECTORY_SEPARATOR . 'AutoViewerGalleryRenderer.php');

class ssch_simpleviewer_viewersconfiguration_testcase extends tx_phpunit_testcase {

    /**
     * Create all neccessary stuff at the beginning
     *
     * @return void
     */
    public function setUp() {

    }

    /**
     * Test the simpleviewer configuration
     */
    public function test_SimpleViewerInitialConfiguration() {
        /* @var $viewer SimpleViewerGalleryRenderer */
        $viewer = t3lib_div::makeInstance('SimpleViewerGalleryRenderer');
        $options =
                array(
                    'title' => 'Title',
                    'textColor' => '#FFFFFF',
                    'frameColor' => '#FFFFFF',
                    'frameWidth' => 20,
                    'galleryStyle' => 'MODERN',
                    'thumbPosition' => 'LEFT',
                    'thumbColumns' => 3,
                    'thumbRows' => 3,
                    'showOpenButton' => TRUE,
                    'showFullscreenButton' => TRUE,
                    'maxImageWidth' => 600,
                    'maxImageHeight' => 600,
                    'useFlickr' => TRUE,
                    'flickrUserName' => 'Name',
                    'flickrTags' => 'Tags',
                    'languageCode' => 'AUTO',
                    'languageList' => 'Previous',
        );
        $this->assertViewerConfiguration($viewer, $options);
    }

    /**
     * Test the postcardviewer configuration
     */
    public function test_PostcardViewerInitialConfiguration() {
        /* @var $viewer PostcardViewerGalleryRenderer */
        $viewer = t3lib_div::makeInstance('PostcardViewerGalleryRenderer');
        $options =
                array(
                    'cellDimension' => 800,
                    'columns' => 8,
                    'zoomOutPerc' => 15,
                    'zoomInPerc' => 100,
                    'frameWidth' => 20,
                    'frameColor' => '0xFFFFFF',
                    'captionColor' => '0xFFFFFF',
                    'enableRightClickOpen' => false
        );
        $this->assertViewerConfiguration($viewer, $options);
    }

    /**
     * Test the autoviewer configuration
     */
    public function test_AutoViewerInitialConfiguration() {
        /* @var $viewer AutoViewerGalleryRenderer */
        $viewer = t3lib_div::makeInstance('AutoViewerGalleryRenderer');
        $options =
                array(
                    'imagePadding' => 8,
                    'frameColor' => '0xFFFFFF',
                    'frameWidth' => 20,
                    'displayTime' => 2,
                    'enableRightClickOpen' => false
        );
        $this->assertViewerConfiguration($viewer, $options);
    }

    /**
     * Test the tiltviewer configuration
     */
    public function test_TiltViewerInitialConfiguration() {
        /* @var $viewer TiltViewerGalleryRenderer */
        $viewer = t3lib_div::makeInstance('TiltViewerGalleryRenderer');
        $options = array();
        $this->assertViewerConfiguration($viewer, $options);
    }

    /**
     * Helper function to assert the viewers specfic configurations
     * @param AbstractRenderer $viewer
     * @param array $configurations
     */
    protected function assertViewerConfiguration(AbstractRenderer $viewer, array $configurations = array()) {
        $expectedKeys = array();
        $expectedKeys = (array_keys($configurations));
        sort($expectedKeys);
        $actualKeys = array();
        $actualKeys = array_keys($viewer->getOptions());
        sort($actualKeys);

        $arrayDiff1 = array_diff($expectedKeys, $actualKeys);
        $arrayDiff2 = array_diff($actualKeys, $expectedKeys);
        $arrayDiff = array_merge($arrayDiff1, $arrayDiff2);

        self::assertSame($expectedKeys, $actualKeys, 'The array are different:' . $arrayDiff);
    }

    /**
     * Try to free as much memory as possible at the end
     *
     * @return void
     */
    public function tearDown() {
        $this->pi1 = null;
    }

}